(****************************************************************)
(*                                                              *)
(*       MidPak Interface Unit v0.1 Beta                        *)
(*       Compiler: TMT Pascal 2.11                              *)
(*       Targets:  MS-DOS 32-bit protected mode only            *)
(*                                                              *)
(*       Copyright (c) 1995,98 TMT Development Corporation      *)
(*       Author: Vadim Bodrov                                   *)
(*                                                              *)
(****************************************************************)

(*
                                IMPORTANT NOTICE

    A  $500  per  product  license  fee  applies  to  all commercial software
 products  distributed  with  ANY DIGPAK and another $500 for MIDPAK drivers.
 That's  a  total  of  $1,000  if  your  product  uses BOTH DIGPAK and MIDPAK
 drivers.  If  you  distribute any commercial title with DIGPAK and/or MIDPAK
 drivers  resident  then  this  license fee applies. To pay a license, simply
 write  a  check for $500 for just DIGPAK $500 for just MIDPAK, or $1,000 for
 both  DIGPAK  and MIDPAK single product distribution license payable to: The
 Audio  Solution,  747  Napa  Lane, St. Charles, MO 63304 with a copy of your
 commerical  product.  You  will  receive a signed license agreement from The
 Audio  Solution shortly thereafter. This license fee applies specifically to
 the  inclusion  with  your distribution disk any of the DIGPAK and/or MIDPAK
 drivers.  These drivers are copyrighted works, created by me, to enhance the
 use  of  sound  and music in DOS based commercial software. The license fees
 collected  are  used  to  maintain  the  drivers  and  keep the BBS running.

 WARNING!!!!!!  You  would  be ill-advised to distribute a commercial product
 containing  either  DIGPAK  and/or  MIDPAK  drivers  without having paid the
 distribution  license  fee.  Since  your  product  would  contain unlicensed
 copyrighted software from The Audio Solution, your product could be required
 to be immediately removed from retail distribution. I doubt this is going to
 be  a  problem.  Clearly  if  your  product  is enhanced by the use of these
 drivers,  your  company can easily afford a nominal license fee of $1,000 in
 exchange  for  getting  the use of several man-years of software engineering
 resources.
*)

(***************************************************************************)
(* Used materials:                                                         *)
(* 'TRICKS OF THE GAME-PROGRAMMING GURUS' by Andre LaMothe, John Ratcliff, *)
(* Mark Seminatore, Denise Tyler, Copyright (C) 1994 Sams Publishing       *)
(***************************************************************************)

 unit MidPak;
 {$w-,r-,q-,i-,t-,x+,v-,a+,oa-,opt+}

 interface

 type TMidiStruct = record
  Addr:    Pointer;
  Size:    LongInt;
  Para:    Word;
 end;

 function  LoadMidPak (MidPak,Adv,Ad: String): Boolean;
 function  UnLoadMidPak: Boolean;
 function  InitMidPak: Boolean;
 function  DeInitMidPak: Boolean;
 function  MidiLoad (Name: String; var MidiFile: TMidiStruct): Boolean;
 procedure MidiFree (var MidiFile: TMidiStruct);
 function  RegisterXMidi (MidiFile: TMidiStruct): Byte;
 function  ReportTriggerCount (var ID: Word): Word;
 procedure ResetTriggerCount;
 function  DigPakAvailable: Boolean;
 function  MidPakClock: LongInt;
 function  MidPakClockAddress: Pointer;
 procedure RemapChannel (Sequence: Word; Physical: Word);
 procedure ResetCallbackCounter;
 procedure MidiSleep;
 procedure MidiAwake;
 procedure PollMidPak;
 function  TriggerCountAddress: Pointer;
 function  EventIDAddress: Pointer;
 function  ReportSequenceNumber: Word;
 function  PlaySequence (SEQ: Word): Boolean;
 procedure ResumePlaying;
 function  SequenceStatus: Word;
 function  RelativeVolume: Word;
 procedure SetRelativeVolume (Volume: Word);
 procedure SegueSequence (SEQ: Word; ACT: Integer);
 procedure StopMidi;

 implementation

 uses DPMI, MDUtil;

 var HaveMidPak: Boolean := false;
     MidPakOn:   Boolean := false;

     MidPakLoad: Pointer;
     MidPakSize: LongInt;
     MidPakPara: Word;
     AdLoad:     Pointer;
     AdSize:     LongInt;
     AdPara:     Word;
     AdvLoad:    Pointer;
     AdvSize:    LongInt;
     AdvPara:    Word;

 var Clock:      Word;
     Trigger:    Word;
     EventID:    Word;
     MidPakExit: Pointer;

 procedure MPInit;
 var Regs: TRmRegs;
 begin
  ClearRmRegs(Regs);
  Regs.BX:=AdvPara;
  Regs.CX:=0;
  Regs.DX:=AdPara;
  Regs.SI:=0;
  Regs.AX:=$710;
  RealModeInt($66,Regs);
 end;

 function LoadMidPak(MidPak,Adv,Ad: String): Boolean;
 begin
  if HaveMidPak then begin
   LoadMidPak:=false; exit;
  end;
  MidPakSize:=SizeOfFile(MidPak);
  AdvSize:=SizeOfFile(Adv);
  AdSize:=SizeOfFile(Ad);
  MidPakPara:=xGlobalDosAlloc(MidPakLoad,MidPakSize+$100);
  if MidPakLoad=nil then begin
   LoadMidPak:=false; exit;
  end;
  AdvPara:=xGlobalDosAlloc(AdvLoad,AdvSize);
  if AdvLoad=nil then begin
   xGlobalDosFree(MidPakLoad);
   LoadMidPak:=false; exit;
  end;
  AdPara:=xGlobalDosAlloc(AdLoad,AdSize);
  if AdLoad=nil then begin
   xGlobalDosFree(MidPakLoad);
   xGlobalDOSFree(AdvLoad);
   LoadMidPak:=false; exit;
  end;
  if not (ReadFromFile(MidPak,Pointer(DWord(MidPakLoad)+$100),0,MidPakSize) and
   ReadFromFile(Adv,AdvLoad,0,AdvSize) and
   ReadFromFile(Ad,AdLoad,0,AdSize)) then begin
    xGlobalDosFree(AdLoad);
    xGlobalDOSFree(AdvLoad);
    xGlobalDosFree(MidPakLoad);
    LoadMidPak:=false;
  end else HaveMidPak:=true;
  LoadMidPak:=HaveMidPak;
 end;

 function UnLoadMidPak: Boolean;
 begin
  if HaveMidPak then begin
   if MidPakOn then DeInitMidPak;
   if MidPakLoad<>nil then begin
    xGlobalDosFree(AdLoad);
    xGlobalDosFree(AdvLoad);
    xGlobalDosFree(MidPakLoad);
    MidPakLoad:=nil;
    AdvLoad:=nil;
    AdLoad:=nil;
    MidPakSize:=0;
    AdvSize:=0;
    AdSize:=0;
    HaveMidPak:=false;
   end;
   UnLoadMidPak:=true;
  end else UnLoadMidPak:=false;
 end;

 function InitMidpak: Boolean;
 var Regs: TRmRegs;
 begin
  if (HaveMidPak) and (xGetStr(MidPakLoad,$103,6)='MIDPAK') then begin
   ClearRmRegs(Regs);
   Regs.CS:=MidPakPara;
   Regs.IP:=$200;
   CallRealModeFar(Regs);
   MidPakOn:=Regs.AX=0;
  end;
  InitMidPak:=MidPakOn;
  if MidPakOn then MPInit;
 end;

 function DeInitMidPak: Boolean;
 var Regs: TRmRegs;
 begin
  if MidPakOn then begin
   StopMidi;
   ClearRmRegs(Regs);
   Regs.CS:=MidPakPara;
   Regs.IP:=$203;
   CallRealModeFar(Regs);
   MidPakOn:=false;
   DeInitMidPak:=true;
  end else DeInitMidPak:=false;
 end;

 function MidiLoad(Name: String; var MidiFile: TMidiStruct): Boolean;
 begin
  MidiFile.Addr:=nil;
  if TestFile(Name) then begin
   MidiFile.Size:=SizeOfFile(Name);
   MidiFile.Para:=xGlobalDosAlloc(MidiFile.Addr,MidiFile.Size);
   if MidiFile.Addr=nil then begin
    MidiLoad:=false;
    exit;
   end;
   if not ReadFromFile(Name,MidiFile.Addr,0,MidiFile.Size) then begin
    xGlobalDOSFree(MidiFile.Addr);
    MidiFile.Addr:=nil;
   end;
  end;
  MidiLoad:=MidiFile.Addr<>nil;
 end;

 procedure MidiFree(var MidiFile: TMidiStruct);
 begin
  if MidiFile.Addr<>nil then begin
   xGlobalDOSFree(MidiFile.Addr);
   MidiFile.Size:=0;
   MidiFile.Addr:=nil;
   MidiFile.Para:=0;
  end;
 end;

 function RegisterXMidi(MidiFile: TMidiStruct): Byte;
 var Regs: TRmRegs;
 begin
  ClearRmRegs(Regs);
  Regs.BX:=0;
  Regs.CX:=MidiFile.Para;
  Regs.SI:=Word(MidiFile.Size);
  Regs.DI:=HiWord(MidiFile.Size);
  Regs.AX:=$704;
  RealModeInt($66,Regs);
  RegisterXMidi:=Regs.AX;
 end;

 function  ReportTriggerCount (var ID: Word): Word; assembler;
      asm
        mov     ax,$707
        int     $66
        mov     edi,[ID]
        mov     [edi],dx
 end;

 procedure ResetTriggerCount; assembler;
      asm
        mov     ax,$708
        int     $66
 end;

 function  DigPakAvailable: Boolean; assembler;
      asm
        mov     ax,$701
        int     $66
 end;

 function MidPakClock: LongInt; assembler;
      asm
        mov     ax,$712
        int     $66
 end;

 function MidPakClockAddress: Pointer;
 var Regs: TRmRegs;
 begin
  if Clock=0 then begin
   ClearRmRegs(Regs);
   Regs.AX:=$712;
   RealModeInt($66,Regs);
   Clock:=AllocateDescriptors(1);
   SetSelectorBaseAddress(Clock,DWord(Regs.CX)*16+Regs.BX);
   SetSelectorLimit(Clock,4);
  end;
  MidPakClockAddress:=Ptr(Clock,0);
 end;

 procedure RemapChannel(Sequence: Word; Physical: Word); assembler;
      asm
        mov     ax,$706
        mov     bx,[Sequence]
        mov     cx,[Physical]
        int     $66
 end;

 procedure ResetCallbackCounter; assembler;
      asm
        mov     ax,$708
        int     $66
 end;

 procedure MidiSleep; assembler;
      asm
        mov     ax,$709
        int     $66
 end;

 procedure MidiAwake; assembler;
      asm
        mov     ax,$70A
        int     $66
 end;

 procedure PollMidPak; assembler;
      asm
        mov     ax,$711
        int     $66
 end;

 function TriggerCountAddress: Pointer;
 var Regs: TRmRegs;
 begin
  if Trigger=0 then begin
   ClearRmRegs(Regs);
   Regs.AX:=$713;
   RealModeInt($66,Regs);
   Trigger:=AllocateDescriptors(1);
   SetSelectorBaseAddress(Trigger,DWord(Regs.DX)*16+Regs.AX);
   SetSelectorLimit(Trigger,4);
  end;
  TriggerCountAddress:=Ptr(Trigger,0);
 end;

 function EventIDAddress: Pointer;
 var Regs: TRmRegs;
 begin
  if EventID=0 then begin
   ClearRmRegs(Regs);
   Regs.AX:=$714;
   RealModeInt($66,Regs);
   EventID:=AllocateDescriptors(1);
   SetSelectorBaseAddress(EventID,DWord(Regs.DX)*16+Regs.AX);
   SetSelectorLimit(EventID,4);
  end;
  EventIDAddress:=Ptr(EventID,0);
 end;

 function ReportSequenceNumber: Word; assembler;
      asm
        mov     ax,$716
        int     $66
 end;

 function  PlaySequence(SEQ: Word): Boolean; assembler;
      asm
        cmp     [MidPakOn],1
        jnz     @Exit
        mov     ax,$702
        mov     bx,[SEQ]
        int     $66
@Exit:
 end;

 procedure ResumePlaying; assembler;
      asm
        mov     ax,$70B
        int     $66
 end;

 function  SequenceStatus: Word; assembler;
      asm
        mov     ax,$70C
        int     $66
 end;

 function  RelativeVolume: Word; assembler;
      asm
        mov     ax,$70E
        int     $66
 end;

 procedure SetRelativeVolume(Volume: Word); assembler;
      asm
        mov     ax,$70F
        mov     bx,[Volume]
        int     $66
  end;

 procedure SegueSequence(SEQ: Word; ACT: Integer); assembler;
      asm
        mov     ax,$703
        mov     bx,[SEQ]
        mov     cx,[ACT]
        int     $66
 end;

 procedure StopMidi; assembler;
      asm
        mov     ax,$705
        int     $66
 end;

 procedure ExitMidPak;
 begin
  if Clock<>0 then FreeDescriptor(Clock);
  if Trigger<>0 then FreeDescriptor(Trigger);
  if EventID<>0 then FreeDescriptor(EventID);
 end;

 begin
  HaveMidPak:=false;
  MidPakOn:=false;
  MidPakExit:=ExitProc;
  ExitProc:=@ExitMidPak;
 end.